import * as React from 'react';
import {AudioFile, getMimeTypeDescription} from '@/utils/blob';
import {formatFileSize, formatNumeral, formatSecondsDurationPrecise} from '@/utils/numeral';
import {useAudioFilePreviewControls} from '@/categories/blob/react/hooks/audio';

export interface AudioFilePreviewProps {
  file: AudioFile;
}

export const AudioFilePreview: React.FC<AudioFilePreviewProps> = ({
  file: f,
}) => {
  const {
    mediaContainerRef,
    playMedia,
    isPlaying,
  } = useAudioFilePreviewControls({ file: f });

  return (
    <div className="flex flex-col gap-4 w-full h-full relative">
      <div className="h-2/5 flex-shrink-0 cursor-pointer relative">
        <div
          ref={mediaContainerRef}
          className="relative h-full w-full"
        />
        <div className="absolute bottom-0 left-0 z-[2]">
          <button
            onClick={playMedia}
          >
            {isPlaying ? '⏸' : '▶'}
          </button>
        </div>
      </div>
      <dl className="h-3/5 flex-shrink-0 m-0 flex flex-col items-end" data-testid="infoBox">
        <div className="w-full">
          <dt className="sr-only">
            Name
          </dt>
          <dd
            className="m-0 w-full text-ellipsis overflow-hidden"
            title={f.name}
          >
            {f.name}
          </dd>
        </div>
        <div className="w-full">
          <dt className="sr-only">
            Type
          </dt>
          <dd
            title={f.type}
            className="m-0 w-full text-ellipsis overflow-hidden"
          >
            {getMimeTypeDescription(f.type, f.name)}
          </dd>
        </div>
        <div className="w-full">
          <dt className="sr-only">
            Size
          </dt>
          <dd
            className="m-0 w-full text-ellipsis overflow-hidden"
            title={`${formatNumeral(f.size ?? 0)} bytes`}
          >
            {formatFileSize(f.size)}
          </dd>
        </div>
        {
          typeof f.metadata?.duration === 'number'
          && (
            <div className="w-full">
              <dt className="sr-only">
                Duration
              </dt>
              <dd
                className="m-0 w-full text-ellipsis overflow-hidden"
                title={`${formatNumeral(f.metadata.duration ?? 0)} seconds`}
              >
                {formatSecondsDurationPrecise(f.metadata.duration)}
              </dd>
            </div>
          )
        }
      </dl>
    </div>
  );
};
