import * as React from 'react';
import WaveSurfer from 'wavesurfer.js';
import {AudioFile} from '@/utils/blob';

export interface UseAudioFilePreviewControlsOptions {
  file: AudioFile;
}

export const useAudioFilePreviewControls = ({ file: f }: UseAudioFilePreviewControlsOptions) => {
  const mediaContainerRef = React.useRef<HTMLDivElement>(null);
  const mediaControllerRef = React.useRef<WaveSurfer>(null);
  const [isPlaying, setIsPlaying] = React.useState(false);

  React.useEffect(() => {
    if (!mediaControllerRef.current) {
      return;
    }

    if (isPlaying) {
      void mediaControllerRef.current.play();
      return;
    }

    mediaControllerRef.current.pause();
  }, [isPlaying]);

  React.useEffect(() => {
    if (!mediaControllerRef.current) {
      return;
    }

    if (!mediaContainerRef.current) {
      return;
    }

    if (typeof f.metadata?.previewUrl !== 'string') {
      return;
    }

    mediaContainerRef.current.innerHTML = '';

    if (f.type === 'audio/mid') {
      return;
    }

    const mediaControllerRefMutable = mediaControllerRef as React.MutableRefObject<WaveSurfer>;
    mediaControllerRefMutable.current = WaveSurfer.create({
      container: mediaContainerRef.current,
      cursorWidth: 0,
      height: mediaContainerRef.current.offsetHeight,
      barWidth: 2,
      barGap: 2,
      barRadius: 1,
    });

    mediaControllerRefMutable.current.on('finish', () => {
      setIsPlaying(false);
      mediaControllerRefMutable.current.seekTo(0);
    });

    // TODO get the preview URL here

    mediaControllerRefMutable.current.load(f.metadata.previewUrl);

    return () => {
      if (!mediaControllerRefMutable.current) {
        return;
      }

      mediaControllerRefMutable.current.destroy();
    }
  }, [f, mediaContainerRef, mediaControllerRef]);

  const playMedia = React.useCallback(() => {
    setIsPlaying((p) => !p);
  }, []);

  return React.useMemo(() => ({
    mediaContainerRef,
    playMedia,
    isPlaying,
  }), [
    mediaContainerRef,
    playMedia,
    isPlaying,
  ]);
};
